/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.SwapListDialog");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");

dojo.require("jazz.ui.internal.SwapList");

dojo.requireLocalization("jazz.ui", "SwapListDialogMessages");


(function() {

/*
 * Aliases 
 */
var SwapList = jazz.ui.internal.SwapList;
var Dialog = jazz.ui.Dialog;

/*
 * Class 
 *
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
dojo.declare("jazz.ui.SwapListDialog",
	[dijit._Widget, dijit._Templated], {

	/*
	 * Properties
	 */	
	/*
	 * Specify the entry field to use when sorting list box entries
	 * Options: "label", "value", "id", "type"
	 */		
	sortField: "label",	
	
	/*  
 	 * Specify icons for selected (left) and unselected (right) bucket 
 	 * entries by providing a map of entry types associated with icon pathes.
 	 * 
 	 * Array iconMap : [Number entryType] = String iconPath
 	 */	
	iconMap: [],
	
	/*  
 	 * Specify the callback function to use when the OK button is clicked. 
 	 */	
	onOK: null,

	/*  
 	 * Specify the callback function to use when either the Cancel button is 
 	 * clicked, the ESC key is pressed or the dialog is closed. 
 	 */	
	onCancel: null,
	
	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/SwapListDialog.html"),
	
	/*
 	 * @param Object args
 	 * 
 	 * @return void
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("jazz.ui", "SwapListDialogMessages");
		this.msgOK = this.messages.msgOK;
		this.msgCancel = this.messages.msgCancel;
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->
		
		this._doIt = false;
		this._dialog = null;
		
		this._multiSelectArea = new SwapList({
			unselectedListTitleText: this.messages.msgAllParameters,
			selectedListTitleText: this.messages.msgSelectedParameters,
			sortField: this.sortField,
			iconMap: this.iconMap
		}); 
		this._contentArea.appendChild(this._multiSelectArea.domNode);
		
		this._connectHandlers();
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	show: function() {
		this._doIt = false;
		this._dialog = new Dialog({
			contentNode: this.domNode,
			primaryTitle: this.messages.msgSelectParameters,
			width: "670px",
			onClose: dojo.hitch(this, this._handleEventDialogClosed) 
		});
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	hide: function() {
		if (this._dialog) {
			this._dialog.close();
			this._dialog.destroy();
			delete this._dialog;
		}
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventDialogClosed: function() {

		if (this._doIt && 
			this.onOK) {
			this.onOK(this);
		} else if (this.onCancel) {
			this.onCancel(this);
		}
		
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_connectHandlers: function() {
		// OK button
		this._clickOKHandler = this.connect(this._okButton, "onclick", this._handleEventOKButtonClicked);
		// Cancel button
		this._clickCancelHandler = this.connect(this._cancelButton, "onclick", this._handleEventCancelButtonClicked);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_disconnectHandlers: function() {
		if (this._clickOKHandler) {
			this.disconnect(this._clickOKHandler);
			delete this._clickOKHandler;
		}
		if (this._clickCancelHandler) {
			this.disconnect(this._clickCancelHandler);
			delete this._clickCancelHandler;
		}
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOKButtonClicked: function(event) {
		dojo.stopEvent(event);
		this._doIt = true;
		this.hide();
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventCancelButtonClicked: function(event) {
		dojo.stopEvent(event);
		this._doIt = false;
		this.hide();	
		
		return true;
	},	

	/*
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * 
 	 * @return void
 	 */	
	setSelectedEntries: function(entries) {
		if (this._multiSelectArea) {
			this._multiSelectArea.setSelectedEntries(entries);
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getSelectedEntries: function() {
		if (!this._multiSelectArea) {
			return null;
		}
		return this._multiSelectArea.getSelectedEntries();
	},

	/*
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * 
 	 * @return void
 	 */	
	setUnselectedEntries: function(entries) {
		if (this._multiSelectArea) {
			this._multiSelectArea.setUnselectedEntries(entries);
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getUnselectedEntries: function() {
		if (!this._multiSelectArea) {
			return null;
		}
		return this._multiSelectArea.getUnselectedEntries();
	},

	/*
 	 * @param boolean show
 	 * 
 	 * @return void
 	 */	
	showLoading: function(show) {
		if (this._multiSelectArea) {
			this._multiSelectArea.showLoading(show);
		}
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		this._disconnectHandlers();
		
		if (this._multiSelectArea) {
			this._multiSelectArea.destroy();
			delete this._multiSelectArea;
		}

		this.inherited(arguments);
	}
	
});

})();
