/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserCALs");

dojo.require("dojox.data.dom");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageUserCALsMessages");

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamContributorSelectionDialog = com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;

function setText(e, text){
	e.appendChild(document.createTextNode(text));
}

var LICENSE_KEY_MANAGEMENT_ACTION = "#action=com.ibm.team.repository.admin.manageLicenses";
var SEARCH_ACTION = "action=com.ibm.team.repository.manageUserCALs";
var PAGE_SIZE = 50;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserCALs", com.ibm.team.repository.web.ui.internal.admin.ItemList, {
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageUserCALsMessages");
	},
	
	postCreate: function() {
		this.header = new WizardStepHeader({ 
			stepTitle: this.messages.calManagement,
			reset: dojo.hitch(this,"reload")
		});
		this.headerArea.appendChild(this.header.domNode);
		setText(this.description, this.messages.pageDescription);
		this.licenseMap = {};
		this.tableHeading.parentNode.removeChild(this.tableHeading);
		this.newItemButtonArea.parentNode.removeChild(this.newItemButtonArea);
		this.tabSection = document.createElement("div");
		this.tabSection.className = "tabSection";
		this.tableControls.appendChild(this.tabSection);
		this.licenseDescription = document.createElement("div");
		this.licenseDescription.className = "licenseDescription";
		this.tableControls.appendChild(this.licenseDescription);
		this.licenseCountArea = document.createElement("div");
		this.licenseCountArea.className = "licenseCountArea";
		this.tableControls.appendChild(this.licenseCountArea);
		this.controlButtonArea = document.createElement("div");
		this.tableControls.appendChild(this.controlButtonArea);
		this.assignButton = document.createElement("button");
		this.connect(this.assignButton,"onclick", function (e) {
			dojo.stopEvent(e);
			var userPicker = new TeamContributorSelectionDialog({ 'disableUsersWithLicense' : this.latestSearch.licenseName });
			this.connect(userPicker,"onAdd","assignLicenses");
		});
		this.assignButton.appendChild(document.createTextNode(this.messages.assignCALs));
		this.controlButtonArea.appendChild(this.assignButton);
		this.setHeaderRow([this.messages.name, this.messages.userId, this.messages.email]);
		this.pageNumberContainer = document.createElement("div");
		this.pageNumberContainer.style.marginTop = "5px";
		this.pageNumberContainer.style.fontWeight = "normal";
		this.pageNumberContainer.style.display = "none";
		this.pageNumberContainer.appendChild(document.createTextNode(dojo.string.substitute(this.messages.showingFirstUsers, [ PAGE_SIZE ])));
		this.tableControls.appendChild(this.pageNumberContainer);
	},
	
	refresh: function () {
		this.header.clearMessages();
		this.clearState();
		var tr = document.createElement("tr");
		var td = document.createElement("td");
		td.appendChild(document.createTextNode(this.messages.searchingForUsers));
		tr.appendChild(td);
		this.tbody.appendChild(tr);
		var srh = new ServiceResponseHandler(this, "getLicenseInfoCallback", "getLicenseInfoErrorCallback");
		LicenseClient.getLicenseInfo(srh);
	},
	
	show: function () {
		this.refresh();
	},
	
	reload: function () {
		this.refresh();
		this.runLatestSearch();
	},
	
	clearState: function () {
		dojox.data.dom.removeChildren(this.tbody);
		this.pageNumberContainer.style.display = "none";
		this.clearMessage();
	},
	
	getLicenseInfoCallback: function (data) {
		dojox.data.dom.removeChildren(this.tabSection);
		if (data.serverLicense != null) {
			this.isTrialLicense = data.serverLicense.expirationTime ? true : false;
		}
		if (data.types != null) {
			var self = this;
			var licenses = LicenseClient.sortCALs(data.types);
			dojo.forEach(licenses, function (license) {
				if (self.defaultLicenseSearch == null) {
					self.defaultLicenseSearch = license.id;
				}
				self.licenseMap[license.id] = license;
				var link = document.createElement("a");
				link.licenseId = license.id;
				link.href = "#" + SEARCH_ACTION + "&licenseType=" + license.id;
				link.className = "tab";
				var topWrapper = document.createElement("div");
				topWrapper.className = "topWrapper";
				var topBorder = document.createElement("div");
				topBorder.className = "topBorder";
				topWrapper.appendChild(topBorder);
				var corners = document.createElement("div");
				corners.className = "corners";
				topWrapper.appendChild(corners);
				link.appendChild(topWrapper);
				var innerWrapper = document.createElement("div");
				innerWrapper.className = "innerWrapper";
				var title = document.createElement("div");
				title.className = "title";
				var licenseInfo = LicenseClient.getClientAccessLicenseInfo(data, license);
				title.appendChild(document.createTextNode(licenseInfo.name));
				innerWrapper.appendChild(title);
				link.appendChild(innerWrapper);
				self.tabSection.appendChild(link);
				self.licenseDescription[license.id] = license.description;
				if (self.licenseCountArea[license.id]) {
					self.licenseCountArea[license.id]["disabled"] = licenseInfo.available === -2;
					self.licenseCountArea[license.id]["disabledReason"] = licenseInfo.available === -2 ? license.disabledReason : "";
					self.licenseCountArea[license.id]["unlimited"] = licenseInfo.available === -1;
					self.licenseCountArea[license.id]["total"] = self.isTrialLicense ? license.max : license.total;
					if (self.latestSearch != null && self.latestSearch.licenseName == license.id) {
						self.showAvailableCount(licenseInfo.available);
					}
				}
				else {
					self.licenseCountArea[license.id] = {
						total: self.isTrialLicense ? license.max : license.total,
						unlimited: licenseInfo.available === -1,
						disabled: licenseInfo.available === -2,
						disabledReason: licenseInfo.available === -2 ? license.disabledReason : ""
					};
				}
				if (self.latestSearch == null) {
					self.runLicenseSearch(license.id, 0);
				}
				self.connect(link, "onclick", function (e) {
					e.target.blur();
				});
			});
			this.updateHeaderLinks();
			this.tableControls.style.display = "";
			this.tbody.parentNode.style.display = "";
		}
		else {
			this.tableControls.style.display = "none";
			this.tbody.parentNode.style.display = "none";
			var message = document.createElement("div");
			message.innerHTML = dojo.string.substitute(this.messages.noCALsInstalled, ["<a href=\"" + LICENSE_KEY_MANAGEMENT_ACTION + "\">", "</a>"]);
			this.displayPendingMessage(message, true);
		}
	},
	
	updateHeaderLinks: function () {
		if (this.tabSection.childNodes != null && this.latestSearch != null) {
			var activeLicense = this.latestSearch.licenseName;
			for (var i=0; i < this.tabSection.childNodes.length; i++) {
				if (this.tabSection.childNodes[i].licenseId == activeLicense) {
					dojo.toggleClass(this.tabSection.childNodes[i], "selected", true);
					this.tabSection.childNodes[i].disabled = true;
				}
				else {
					dojo.toggleClass(this.tabSection.childNodes[i], "selected", false);
					this.tabSection.childNodes[i].disabled = false;
				}
			}
			this.showLicenseDescription(activeLicense);
			this.updateLicenseButton(activeLicense);
			this.showAvailableCount(activeLicense);
			this.displayMessage("");
		}
	},
	
	getLicenseInfoErrorCallback: function (err) {
		this.header.displayServiceCallbackErrorMessage(err);
	},
	
	runLatestSearch: function () {
		if (this.latestSearch != null) {
			this.runLicenseSearch(this.latestSearch.licenseName, this.latestSearch.pageNum);
		}
	},
	
	runLicenseSearch: function (licenseName, pageNum) {
		licenseName = licenseName || this.defaultLicenseSearch;
		this.showLicenseDescription(licenseName);
		this.updateLicenseButton(licenseName);
		if (licenseName != null) {
			this.latestSearch = {
				type: "license",
				licenseName: licenseName,
				pageNum: pageNum
			};
			this.updateHeaderLinks();
			this.latestSearchTerm = licenseName;
			this.latestSearchType = "licenseType";
			this.queryPage = pageNum || 0;
			this.updatePagerDivLinks();
			dojox.data.dom.removeChildren(this.tbody);
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.appendChild(document.createTextNode(this.messages.searchingForUsers));
			tr.appendChild(td);
			this.tbody.appendChild(tr);
			var srh = new ServiceResponseHandler(this, "showUsers", "handleError");
			AdminClient.getContributors({searchTerm: licenseName, isLicenseSearch: true, pageSize: PAGE_SIZE, hideGuest: true, hideUnassigned: true, pageNum: pageNum}, srh);
		}
	},
	
	showLicenseDescription: function (licenseId) {
		dojox.data.dom.removeChildren(this.licenseDescription);
		this.licenseDescription.appendChild(document.createTextNode(this.licenseDescription[licenseId]));
	},
	
	updateLicenseButton: function (licenseId) {
		dojox.data.dom.removeChildren(this.assignButton);
		if (this.licenseMap[licenseId]) {
			this.assignButton.appendChild(document.createTextNode(dojo.string.substitute(this.messages.assignLicenses, [ this.licenseMap[licenseId].name ])));
		}
	},
	
	showAvailableCount: function (licenseId) {
		dojox.data.dom.removeChildren(this.licenseCountArea);
		if (this.licenseCountArea[licenseId] && this.licenseCountArea[licenseId]["total"] != null &&
				this.licenseCountArea[licenseId]["used"] != null) {
			var unlimited = this.licenseCountArea[licenseId]["unlimited"];
			if (!unlimited) {
				var available = this.licenseCountArea[licenseId]["total"] - this.licenseCountArea[licenseId]["used"];
			}
			var disabled = this.licenseCountArea[licenseId]["disabled"];
			if (disabled) {
				var disabledReason = this.licenseCountArea[licenseId]["disabledReason"];
				this.displayWarningMessage(dojo.string.substitute(this.messages.calDisabled, [ disabledReason ]));
				// set a flag if this license is disabled because it is assigned to more users than are allowed
				this.isOverAssigned = disabledReason != null && disabledReason.indexOf("CRJAZ0969I") === 0;
			}
			else {
				this.isOverAssigned = false;
				if (!unlimited) {
					this.licenseCountArea.appendChild(document.createTextNode(dojo.string.substitute(this.messages.licensesAvailable, [available])));
				}
			}
			if (!unlimited && (!available || disabled)) {
				this.assignButton.disabled = true;
			}
			else {
				this.assignButton.disabled = false;
			}
		}
	},
	
	showUsers: function (resultSet) {
		dojox.data.dom.removeChildren(this.tbody);
		if (this.pagerDiv == null) {
			this.createPagerDiv();
		}
		if (this.licenseCountArea[this.latestSearch.licenseName]) {
			this.licenseCountArea[this.latestSearch.licenseName]["used"] = resultSet.count;
			this.showAvailableCount(this.latestSearch.licenseName);
		}
		else {
			this.licenseCountArea[this.latestSearch.licenseName] = { used: resultSet.count };
		}
		if (resultSet == null || resultSet.elements == null || resultSet.elements.length == 0) {
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.appendChild(document.createTextNode(this.messages.noMatchingUsers));
			td.colSpan = 3;
			tr.appendChild(td);
			tr.onclick = function (e) {
				dojo.stopEvent(e);
			};
			this.tbody.appendChild(tr);
			dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
			dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
			dojox.data.dom.removeChildren(this.pagerDiv.rangeSpan);
			dojox.data.dom.removeChildren(this.pagerDiv.totalSpan);
			this.pagerDiv.rangeSpan.appendChild(document.createTextNode("0"));
			this.pagerDiv.totalSpan.appendChild(document.createTextNode("0"));
		}
		else {
			dojox.data.dom.removeChildren(this.pagerDiv.rangeSpan);
			dojox.data.dom.removeChildren(this.pagerDiv.totalSpan);
			if (resultSet.count > PAGE_SIZE) {
				var lowerBound = (this.queryPage * PAGE_SIZE) + 1;
				var upperBound = (this.queryPage + 1) * PAGE_SIZE;
				if (this.queryPage == 0) {
					dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", false);
				}
				if (upperBound >= resultSet.count) {
					upperBound = resultSet.count;
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", false);
				}
				this.pagerDiv.rangeSpan.appendChild(document.createTextNode(lowerBound + " - " + upperBound));
			}
			else {
				dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				this.pagerDiv.rangeSpan.appendChild(document.createTextNode("1 - " + resultSet.count));
			}
			this.pagerDiv.totalSpan.appendChild(document.createTextNode(resultSet.count));
			dojo.forEach(resultSet.elements, dojo.hitch(this, function(user){
				var nameNode = document.createElement("div");
				
				var unassignIconAnchor = document.createElement("a");
				unassignIconAnchor.href = '#';
				unassignIconAnchor.className = "archiveIcon";
					
				var unassignLicenseIcon = document.createElement("img");
				unassignLicenseIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");
				unassignLicenseIcon.title = this.messages.unassignLicense;
				unassignLicenseIcon.alt = this.messages.unassignLicense;
				unassignLicenseIcon.itemId = user.itemId;
				unassignLicenseIcon.userId = user.userId;
				
				this.connect(unassignLicenseIcon,"onmouseover", function (e) {
					e.target.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove-hov.gif");
				});
				this.connect(unassignLicenseIcon, "onmouseout", function (e) {
					e.target.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");
				});
				this.connect(unassignLicenseIcon, "onclick", function (e) {
					dojo.stopEvent(e);
					this.unassignLicense(e.target.userId);
				});
				
				this.connect(unassignIconAnchor, "onclick", function (e) {
					dojo.stopEvent(e);
					this.unassignLicense(e.target.firstChild.userId);
				});
				this.connect(unassignIconAnchor, "onfocus", function (e) {
					var row = e.target.parentNode.parentNode.parentNode;
					if (row && (!row.collapsable || e.target.tagName === "A")) {
						dojo.toggleClass(row, "selected", true);
					}
					e.target.firstChild.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove-hov.gif");
				});
				this.connect(unassignIconAnchor, "onblur", function(e) {
					var row = e.target.parentNode.parentNode.parentNode;
					if (row && (!row.collapsable || e.target.tagName === "A")) {
						dojo.toggleClass(row, "selected", false);
					}
					e.target.firstChild.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");
				});
					
				unassignIconAnchor.appendChild(unassignLicenseIcon);

				nameNode.appendChild(unassignIconAnchor);					
				nameNode.appendChild(document.createTextNode(user.name));

				var row = this.addRow(null, [nameNode, user.userId, user.emailAddress]);
				//row.cells[0].appendChild(unassignIconAnchor);
				dojo.toggleClass(row.parentNode,"userTable",true);
			}));
		}
	},
	
	assignLicenses: function (newUsers) {
		if (newUsers != null) {
			var userIds = [];
			for (var i=0; i < newUsers.length; i++) {
				var user = newUsers[i];
				userIds.push(user.userId);
			}
			var args = {
				jsonUserIds: dojo.toJson(userIds),
				licenseId: this.latestSearch.licenseName
			};
			var srh = new ServiceResponseHandler(this, "assignLicensesCallback", "assignLicensesErrorCallback");
			LicenseClient.assignLicenseMultiple(srh, args);
		}
	},
	
	assignLicensesCallback: function (response) {
		this.runLatestSearch();
		if (response != null && response != "") {
			var errors = dojo.fromJson(response);
			var div = document.createElement("div");
			div.appendChild(document.createTextNode(dojo.string.substitute(this.messages.errorAssigning, [ "" ])));
			div.appendChild(document.createElement("br"));
			for (var i=0; i < errors.length; i++) {
				div.appendChild(document.createElement("br"));
				div.appendChild(document.createTextNode(errors[i].userId + ": " + errors[i].message));
			}
			this.displayErrorMessage(div, true);
		}
		else {
			this.displayMessage("");
		}
	},
	
	assignLicensesErrorCallback: function (err) {
		this.header.displayServiceCallbackErrorMessage(err);
	},
	
	unassignLicense: function (itemId) {
		var srh = new ServiceResponseHandler(this, "unassignLicenseCallback", "unassignLicenseErrorCallback");
		LicenseClient.unassignLicense(srh, { userId: itemId, licenseId: this.latestSearch.licenseName });
	},
	
	unassignLicenseCallback: function (response) {
		if (this.isOverAssigned) {
			this.reload();
		}
		else {
			this.runLatestSearch();
			this.displayMessage("");
		}
	},
	
	unassignLicenseErrorCallback: function (err) {
		this.header.displayServiceCallbackErrorMessage(err);
	},
	
	createPagerDiv: function () {
		this.pagerDiv = document.createElement("div");
		dojo.toggleClass(this.pagerDiv, "pager", true);
		var prevSpan = document.createElement("span");
		var prevLink = document.createElement("a");
		prevLink.appendChild(document.createTextNode(this.messages.previous));
		prevSpan.appendChild(prevLink);
		var disabledPrev = document.createElement("span");
		disabledPrev.appendChild(document.createTextNode(this.messages.previous));
		dojo.toggleClass(disabledPrev, "disabledText", true);
		prevSpan.appendChild(disabledPrev);
		this.pagerDiv.appendChild(prevSpan);
		this.pagerDiv.prevSpan = prevSpan;
		this.pagerDiv.prevLink = prevLink;
		this.pagerDiv.appendChild(document.createTextNode(" | "));
		var rangeSpan = document.createElement("span");
		dojo.toggleClass(rangeSpan, "bold", true);
		this.pagerDiv.appendChild(rangeSpan);
		this.pagerDiv.rangeSpan = rangeSpan;
		this.pagerDiv.appendChild(document.createTextNode(" " + this.messages.of + " "));
		var totalSpan = document.createElement("span");
		dojo.toggleClass(totalSpan, "bold", true);
		this.pagerDiv.appendChild(totalSpan);
		this.pagerDiv.totalSpan = totalSpan;
		this.pagerDiv.appendChild(document.createTextNode(" | "));
		var nextSpan = document.createElement("span");
		var nextLink = document.createElement("a");
		nextLink.appendChild(document.createTextNode(this.messages.next));
		nextSpan.appendChild(nextLink);
		var disabledNext = document.createElement("span");
		disabledNext.appendChild(document.createTextNode(this.messages.next));
		dojo.toggleClass(disabledNext, "disabledText", true);
		nextSpan.appendChild(disabledNext);
		this.pagerDiv.appendChild(nextSpan);
		this.pagerDiv.nextSpan = nextSpan;
		this.pagerDiv.nextLink = nextLink;
		this.tableControls.appendChild(this.pagerDiv);
		this.updatePagerDivLinks();
		this.connect(prevLink,"onclick",function (e) {
			dojo.stopEvent(e);
			document.location.href = e.target.href;
		});
		this.connect(nextLink,"onclick",function (e) {
			dojo.stopEvent(e);
			document.location.href = e.target.href;
		});
	},
	
	updatePagerDivLinks: function () {
		if (this.pagerDiv != null) {
			this.pagerDiv.prevLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage - 1);
			this.pagerDiv.nextLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage + 1);
		}
	}
});

})();